﻿
// MainFrm.cpp : implementation of the CMainFrame class
//

#include "pch.h"
#include "framework.h"
#include "MfcEdgeDemo.h"
#include "MfcEdgeDemoView.h"

#include "MainFrm.h"
#include "Messages.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_BACKGROUND_COLOR, &CMainFrame::OnBnClickedButtonChangeBackgroundColor)
	ON_BN_CLICKED(IDC_BUTTON_COPY_SELECTION, &CMainFrame::OnBnClickedButtonCopySelection)
	ON_BN_CLICKED(IDC_BUTTON_TEST_POST, &CMainFrame::OnBnClickedButtonTestPost)
	ON_BN_CLICKED(IDC_BUTTON_BACK, &CMainFrame::OnBnClickedButtonBack)
	ON_BN_CLICKED(IDC_BUTTON_FORWARD, &CMainFrame::OnBnClickedButtonForward)
	ON_BN_CLICKED(IDC_BUTTON_RELOAD, &CMainFrame::OnBnClickedButtonReload)
	ON_MESSAGE(MSG_NAVIGATE, &CMainFrame::OnNavigateToUrl)
	ON_COMMAND(IDC_BUTTON_RELOAD, &CMainFrame::OnButtonReload)
	ON_UPDATE_COMMAND_UI(IDC_BUTTON_RELOAD, &CMainFrame::OnUpdateButtonReload)
	ON_COMMAND(IDC_BUTTON_SHOWPRINTUI, &CMainFrame::OnButtonShowPrintUI)
	ON_COMMAND(IDC_BUTTON_PRINT_TO_PDF, &CMainFrame::OnButtonPrintToPdf)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

// CMainFrame construction/destruction

CMainFrame::CMainFrame() noexcept
{
	// TODO: add member initialization code here
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_wndDlgBar.Create(this, IDR_MAINFRAME, CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0("Failed to create dialogbar\n");
		return -1;		// fail to create
	}

	if (!m_wndReBar.Create(this) || !m_wndReBar.AddBar(&m_wndDlgBar))
	{
		TRACE0("Failed to create rebar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}
	m_wndStatusBar.SetIndicators(indicators, sizeof(indicators)/sizeof(UINT));
	m_wndStatusBar.ShowWindow(SW_HIDE);

	// TODO: Remove this if you don't want tool tips

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}

// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}
#endif //_DEBUG


// CMainFrame message handlers

void CMainFrame::OnBnClickedButtonBack()
{
	auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	if (view != nullptr)
		view->NavigateBack();
}

void CMainFrame::OnBnClickedButtonTestPost()
{
	auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	if (view != nullptr)
	{
		CString content;

		CString headers = L"Authorization:Basic ZGVtbzpkZW1v\r\nUser-Agent:WebView2 Demo";

		view->NavigatePost(L"https://postman-echo.com/post", content, headers);
	}
}

void CMainFrame::OnBnClickedButtonForward()
{
   auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
   if (view != nullptr)
      view->NavigateForward();
}


void CMainFrame::OnBnClickedButtonReload()
{
   auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
   if (view != nullptr)
      view->ReloadPage();
}

LRESULT CMainFrame::OnNavigateToUrl(WPARAM wParam, LPARAM lParam)
{
	auto url = reinterpret_cast<LPCTSTR>(lParam);
   auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
   if (view != nullptr)
		view->NavigateTo(url, [this, view]() {
			m_wndDlgBar.SetDlgItemText(IDC_EDIT_URL, view->GetCurrentUrl());
		});

	return 0;
}

void CMainFrame::OnButtonReload()
{
	auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	if (view->IsNavigating())
		view->StopNavigation();
	else
		view->ReloadPage();
}

void CMainFrame::OnUpdateButtonReload(CCmdUI* pCmdUI)
{
   auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	pCmdUI->SetText(view != nullptr && view->IsNavigating() ? L"🞪" : L"⭯");
}

void CMainFrame::OnBnClickedButtonCopySelection()
{
	auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	if (view != nullptr)
	{
		view->GetSelection([](CString const& text) {
			AfxMessageBox(L"Selected text: " + text, MB_OK);
			});		
	}
}

void CMainFrame::OnBnClickedButtonChangeBackgroundColor()
{
	auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	if (view != nullptr)
	{
		view->ExecuteScript(L"document.body.style.backgroundColor = \"red\"");
	}
}

void CMainFrame::OnButtonShowPrintUI()
{
	auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	if (view != nullptr)
	{
		CButton* check = (CButton*)m_wndDlgBar.GetDlgItem(IDC_CHECK_SYSTEM_PRINTER);
		view->ShowPrintUI(check->GetCheck() == BST_CHECKED);
	}
}


void CMainFrame::OnButtonPrintToPdf()
{
	auto view = dynamic_cast<CMfcEdgeDemoView*>(GetActiveView());
	if (view != nullptr)
	{
		if (view->IsPrintToPdfInProgress())
		{
			::MessageBox(
				nullptr,
				L"A PDF file is currently printing. Please wait until the operation completes.",
				L"Warning",
				MB_OK|MB_ICONEXCLAMATION);

			return;
		}

		CButton* check = (CButton*)m_wndDlgBar.GetDlgItem(IDC_CHECK_LANDSCAPE);

		view->PrintToPDF(check->GetCheck() == BST_CHECKED, [](bool const isSuccessful, CString path) {
			if (isSuccessful)
			{
				auto result = ::MessageBox(
					nullptr,
					L"Print to PDF succeeded. Would you like to open the document?",
					L"Print to PDF completed",
					MB_YESNO);
				
				if (result == IDYES)
				{
					ShellExecute(0, 0, path, 0, 0, SW_SHOW);
				}
			}
			else
			{
				::MessageBox(
					nullptr,
					L"Print to PDF failed",
					L"Print to PDF completed",
					MB_OK);
			}
		});
	}
}
